/** \file main.c
	\author Tomasz Ostrowski
	\date August 2007
	
	Main file of SHT11 sensor library test project.
	Compiler: avr-gcc (WinAVR 20060421)
	Based on Sensirion sample code app note, rev. 2.05
	MCU: ATmega8, 4MHz crystal, Vcc = 5V (important for SHT11 calculations)
	sensor: SHT11, DATA with external 10k pullup connected to PD4, SCK connected to PD3
	MCU <-> PC: UART
	HFUSE/LFUSE: 0xC9/0x9F
*/

#include "main.h"	//IMPORTANT: it contains F_CPU definition, so include it before delay.h
#include <avr/wdt.h>
#include <stdlib.h>
#include <avr/io.h>
#include <avr/pgmspace.h>
#include <util/delay.h>
#include <avr/interrupt.h>

#include "24c64.c"
#include "24c64.h"


#include "HD44780.h"

#include "SHTxx.h"


#ifndef F_CPU
#define F_CPU           4000000                   // processor clock frequency
#warning kein F_CPU definiert
#endif



#define HMaximum 99  // Max  99 Prozent
#define HMinimum 0    // Min 0  Prozent
#define TMaximum 99  // Max  99 Grad
#define TMinimum -9    // Min 0  Grad


#define KEY_DDR         DDRB
#define KEY_PORT        PORTB
#define KEY_PIN         PINB
#define KEY0            2
#define KEY1            3
#define KEY2            4
#define KEY3            5
#define ALL_KEYS        (1<<KEY0 | 1<<KEY1 | 1<<KEY2 | 1<<KEY3)


#define REPEAT_MASK     (1<<KEY0 | 1<<KEY1)       // repeat: key0, key1
#define REPEAT_START    50                        // after 500ms
#define REPEAT_NEXT     20                        // every 200ms


  uint8_t mcusr_mirror __attribute__ ((section (".noinit")));

    void get_mcusr(void) \
      __attribute__((naked)) \
      __attribute__((section(".init3")));
    void get_mcusr(void)
    {
      mcusr_mirror = MCUSR;
       MCUSR = 0;
       wdt_disable();
    }


volatile uint8_t key_state;                                // debounced and inverted key state:
                                                           // bit = 1: key pressed
volatile uint8_t key_press;                                // key press detect
 
volatile uint8_t key_rpt;                                  // key long press and repeat


 
ISR( TIMER0_OVF_vect )                            // every 10ms
{
  static uint8_t ct0, ct1, rpt;
  uint8_t i;
 
  TCNT0 = (uint8_t)(int16_t)-(F_CPU / 1024 * 10e-3 + 0.5);  // preload for 10ms
 
  i = key_state ^ ~KEY_PIN;                       // key changed ?
  ct0 = ~( ct0 & i );                             // reset or count ct0
  ct1 = ct0 ^ (ct1 & i);                          // reset or count ct1
  i &= ct0 & ct1;                                 // count until roll over ?
  key_state ^= i;                                 // then toggle debounced state
  key_press |= key_state & i;                     // 0->1: key press detect
 
  if( (key_state & REPEAT_MASK) == 0 )            // check repeat function
     rpt = REPEAT_START;                          // start delay
  if( --rpt == 0 ){
    rpt = REPEAT_NEXT;                            // repeat delay
    key_rpt |= key_state & REPEAT_MASK;
  }
}
 
///////////////////////////////////////////////////////////////////
//
// check if a key has been pressed. Each pressed key is reported
// only once
//
uint8_t get_key_press( uint8_t key_mask )
{
  cli();                                          // read and clear atomic !
  key_mask &= key_press;                          // read key(s)
  key_press ^= key_mask;                          // clear key(s)
  sei();
  return key_mask;
}
 
///////////////////////////////////////////////////////////////////
//
// check if a key has been pressed long enough such that the
// key repeat functionality kicks in. After a small setup delay
// the key is reported being pressed in subsequent calls
// to this function. This simulates the user repeatedly
// pressing and releasing the key.
//
uint8_t get_key_rpt( uint8_t key_mask )
{
  cli();                                          // read and clear atomic !
  key_mask &= key_rpt;                            // read key(s)
  key_rpt ^= key_mask;                            // clear key(s)
  sei();
  return key_mask;
}
 
///////////////////////////////////////////////////////////////////
//
// check if a key is pressed right now
//
uint8_t get_key_state( uint8_t key_mask )
 
{
  key_mask &= key_state;
  return key_mask;
}
 
///////////////////////////////////////////////////////////////////
//
uint8_t get_key_short( uint8_t key_mask )
{
  cli();                                          // read key state and key press atomic !
  return get_key_press( ~key_state & key_mask );
}
 
///////////////////////////////////////////////////////////////////
//
uint8_t get_key_long( uint8_t key_mask )
{
  return get_key_press( get_key_rpt( key_mask ));
}




 void wait(void)
{
	int i;

//  for(i = 0; i < 1000; i++)
	for(i = 0; i < 50; i++)
		_delay_ms(1);
}


int main(void)
{

  wdt_enable(WDTO_8S);

  // Configure debouncing routines
  KEY_DDR &= ~ALL_KEYS;                // configure key port for input
  KEY_PORT |= ALL_KEYS;                // and turn on pull up resistors
 
//  TCCR0 = (1<<CS02)|(1<<CS00);         // divide by 1024                 (ATMEGA8)
  TCCR0B = (1<<CS02)|(1<<CS00);         // divide by 1024                (ATMEGA168)

  TCNT0 = (uint8_t)(int16_t)-(F_CPU / 1024 * 10e-3 + 0.5);  // preload for 10ms
//  TIMSK |= 1<<TOIE0;                   // enable timer interrupt         (ATMEGA8)
  TIMSK0 |= (1<<TOIE0);                   // enable timer interrupt          (ATMEGA168)

 
  sei();
  
  

    int t  = 25;
    int h  = 70;
	int s  = 1;
	int o  = 1;
	int u  = 1;
	int a  = 0;
	int au = 1;
	

	value humid_val,temp_val;
	unsigned char checksum, error = 0;
	static int nr = 0;
	char nr_tab[1];
	char buf[5];
	
	s_connectionreset();
	
	lcd_init(LCD_DISP_ON);
	
	lcd_clrscr();

	sei();

	while (1) 
	{
		
/*		//according to app note SHTxx should not be active more than 10% of the time to reduse self-heating to < 0.1 deg				
		//also, no command should be sent to SHT for 11ms after power-up
		for(unsigned char i=0; i<100; i++)
			_delay_ms(10);	// (10) WARNING, max param 262.14ms/F_CPU


        //----------wait approx. 0.8s to avoid heating up SHTxx------------------------------      
        for (unsigned char i=0;i<100;i++);     //(be sure that the compiler doesn't eliminate this line!)
	        _delay_ms(10);	                          // (10) WARNING, max param 262.14ms/F_CPU
			
*/
		
		//WARNING! s_measure may take up to 2s (max timeout), so you'll probably
		//have to rewrite this function into non-blocking code if your application does sth time-critical
 		error+=s_measure((unsigned char*) &temp_val.i,&checksum,TEMP);
 		error+=s_measure((unsigned char*) &humid_val.i,&checksum,HUMID);
		
		
		// "s","o" und "u" muss man erstmal im EEprom auf "1" schreiben, sonst steht nur ein "T:" im Display (z.B. mit: "s = 1;")
		
		
   		t = EEReadByte(0);
   		h = EEReadByte(1);
		s = EEReadByte(2);
		o = EEReadByte(3);
		u = EEReadByte(4);
		a = EEReadByte(5);
		
		
		
/*		s = EEWriteByte(2, 1);      // Das ist nur zum intitialisieren eines jungfulichen EEproms!!!!!
		o = EEWriteByte(3, 1);      // Das ist nur zum intitialisieren eines jungfulichen EEproms!!!!!
		u = EEWriteByte(4, 1);      // Das ist nur zum intitialisieren eines jungfulichen EEproms!!!!!
	   au = EEWriteByte(6, 1);      // Das ist nur zum intitialisieren eines jungfulichen EEproms!!!!!
	   
		a = EEWriteByte(5, 0);      // Das ist zum Abnullen des Fehlerspeichers!!!!!
*/
    	
		
	if(au == (-1))
	{
	  wdt_reset();

	 lcd_clrscr();
	 lcd_gotoxy(0,0);		  
     lcd_puts("Error-Messungen:");	   
     lcd_gotoxy(0,1);
	 itoa(a, buf, 10);
	 lcd_puts(buf);
    }

	if(au == (1))
    {
		if(error != 0)
		{ 	
		   s_connectionreset(); 
		   a = a + 1;
		   a = EEWriteByte(5, a);
		   lcd_clrscr();
		   lcd_gotoxy(0,0);		  
		   lcd_puts("MESSUNG ERROR");	   
		   itoa(nr, nr_tab, 10);
		   lcd_gotoxy(0, 1);
		   lcd_puts(nr_tab);
		   nr++;
		   wait();
		   
		   DDRB |= (1 << DDB1); // Setzen
		   _delay_ms(1000);	//
		   DDRB &= ~(1 << DDB1); // Rcksetzen
		}
		else
		{
 		   wdt_reset();
		   
		   lcd_clrscr();
		   
		   humid_val.f=(float)humid_val.i;
		   temp_val.f=(float)temp_val.i;
		   calc_sht11(&humid_val.f,&temp_val.f);
		   humid_val.i=(int32_t)(humid_val.f*100);
		   temp_val.i=(int32_t)(temp_val.f*100);
		   
		   lcd_puts("T:");
		   
		   if(temp_val.i < 0)
		    {
		   		lcd_gotoxy(3,0);
				lcd_puts("-");
			}
			
			
			temp_val.i = abs(temp_val.i);
			
		if (( s == (1)) && ( o == (-1)) && ( u == (-1)))
		   {
		   // Anzeige Temperatur
		  if(temp_val.i/100 >= 10)
           {
		   lcd_gotoxy(4,0);
		   itoa(temp_val.i/100, buf, 10);
		   lcd_puts(buf);
		   lcd_gotoxy(6,0);
		   lcd_puts(".");
		   		if((temp_val.i%100) < 10)
				{
				lcd_puts("0");
		        itoa(temp_val.i%100, buf, 10);
		        lcd_puts(buf);
		        }
				else
				{
		        itoa(temp_val.i%100, buf, 10);
		        lcd_puts(buf);
				}
		   lcd_gotoxy(10,0);
		   lcd_puts("C");
		   lcd_gotoxy(12,0);
		   lcd_puts("*<");
		   lcd_gotoxy(14,0);
		   itoa(t, buf, 10);
		   lcd_puts(buf);
		   }
		  if(temp_val.i/100 <= 9)
           {
		   lcd_gotoxy(5,0);
		   itoa(temp_val.i/100, buf, 10);
		   lcd_puts(buf);
		   lcd_gotoxy(6,0);
		   lcd_puts(".");
		   		if((temp_val.i%100) < 10)
				{
				lcd_puts("0");
		        itoa(temp_val.i%100, buf, 10);
		        lcd_puts(buf);
		        }
				else
				{
		        itoa(temp_val.i%100, buf, 10);
		        lcd_puts(buf);
				}
		   lcd_gotoxy(10,0);
		   lcd_puts("C");
		   lcd_gotoxy(12,0);
		   lcd_puts("*<");
		   lcd_gotoxy(14,0);
		   itoa(t, buf, 10);
		   lcd_puts(buf);
		   }
		   // Anzeige Humidity
		   lcd_gotoxy(0,1);
		   humid_val.i = abs(humid_val.i);
		   lcd_puts("H:");
		   itoa(humid_val.i/100, buf, 10);
		   lcd_gotoxy(4,1);
		   lcd_puts(buf);
		   lcd_gotoxy(6,1);
		   lcd_puts(buf);
		   lcd_gotoxy(6,1);
		   lcd_puts(".");
		   		if((humid_val.i%100) < 10)
				{
				lcd_puts("0");
		        itoa(humid_val.i%100, buf, 10);
		        lcd_puts(buf);
		        }
				else
				{
		        itoa(humid_val.i%100, buf, 10);
		        lcd_puts(buf);
				}
		   lcd_gotoxy(10,1);
		   lcd_puts("%");
		   lcd_gotoxy(13,1);
		   lcd_puts("<");
		   lcd_gotoxy(14,1);
		   itoa(h, buf, 10);
		   lcd_puts(buf);
    	   }
		   
		if (( s == (1)) && ( o == (1)) && ( u == (1)))
		   {
		   // Anzeige Temperatur
		  if(temp_val.i/100 >= 10)
           {
		   lcd_gotoxy(4,0);
		   itoa(temp_val.i/100, buf, 10);
		   lcd_puts(buf);
		   lcd_gotoxy(6,0);
		   lcd_puts(".");
		   		if((temp_val.i%100) < 10)
				{
				lcd_puts("0");
		        itoa(temp_val.i%100, buf, 10);
		        lcd_puts(buf);
		        }
				else
				{
		        itoa(temp_val.i%100, buf, 10);
		        lcd_puts(buf);
				}
		   lcd_gotoxy(10,0);
 		   lcd_puts("C");
		   lcd_gotoxy(12,0);
		   lcd_puts("*>");
		   lcd_gotoxy(14,0);
		   itoa(t, buf, 10);
		   lcd_puts(buf);
		   }
		  if(temp_val.i/100 <= 9)
           {
		   lcd_gotoxy(5,0);
		   itoa(temp_val.i/100, buf, 10);
		   lcd_puts(buf);
		   lcd_gotoxy(6,0);
		   lcd_puts(".");
		   		if((temp_val.i%100) < 10)
				{
				lcd_puts("0");
		        itoa(temp_val.i%100, buf, 10);
		        lcd_puts(buf);
		        }
				else
				{
		        itoa(temp_val.i%100, buf, 10);
		        lcd_puts(buf);
				}
		   lcd_gotoxy(10,0);
		   lcd_puts("C");
		   lcd_gotoxy(12,0);
		   lcd_puts("*>");
		   lcd_gotoxy(14,0);
		   itoa(t, buf, 10);
		   lcd_puts(buf);
		   }
		   
        // Anzeige Humidity
		   lcd_gotoxy(0,1);
		   humid_val.i = abs(humid_val.i);
		   lcd_puts("H:");
		   itoa(humid_val.i/100, buf, 10);
		   lcd_gotoxy(4,1);
		   lcd_puts(buf);
		   lcd_gotoxy(6,1);
		   lcd_puts(buf);
		   lcd_gotoxy(6,1);
		   lcd_puts(".");
		   		if((humid_val.i%100) < 10)
				{
				lcd_puts("0");
		        itoa(humid_val.i%100, buf, 10);
		        lcd_puts(buf);
		        }
				else
				{
		        itoa(humid_val.i%100, buf, 10);
		        lcd_puts(buf);
				}
		   lcd_gotoxy(10,1);
		   lcd_puts("%");
		   lcd_gotoxy(13,1);
		   lcd_puts(">");
		   lcd_gotoxy(14,1);
		   itoa(h, buf, 10);
		   lcd_puts(buf);
    	   }

		if (( s == (-1)) && ( o == (-1)) && ( u == (-1)))
		   {
		   // Anzeige Temperatur
		  if(temp_val.i/100 >= 10)
           {
		   lcd_gotoxy(4,0);
		   itoa(temp_val.i/100, buf, 10);
		   lcd_puts(buf);
		   lcd_gotoxy(6,0);
		   lcd_puts(".");
		   		if((temp_val.i%100) < 10)
				{
				lcd_puts("0");
		        itoa(temp_val.i%100, buf, 10);
		        lcd_puts(buf);
		        }
				else
				{
		        itoa(temp_val.i%100, buf, 10);
		        lcd_puts(buf);
				}
		   lcd_gotoxy(10,0);
		   lcd_puts("C");
		   lcd_gotoxy(13,0);
		   lcd_puts("<");
		   lcd_gotoxy(14,0);
		   itoa(t, buf, 10);
		   lcd_puts(buf);
		   }
		  if(temp_val.i/100 <= 9)
           {
		   lcd_gotoxy(5,0);
		   itoa(temp_val.i/100, buf, 10);
		   lcd_puts(buf);
		   lcd_gotoxy(6,0);
		   lcd_puts(".");
		   		if((temp_val.i%100) < 10)
				{
				lcd_puts("0");
		        itoa(temp_val.i%100, buf, 10);
		        lcd_puts(buf);
		        }
				else
				{
		        itoa(temp_val.i%100, buf, 10);
		        lcd_puts(buf);
				}
		   lcd_gotoxy(10,0);
		   lcd_puts("C");
		   lcd_gotoxy(13,0);
		   lcd_puts("<");
		   lcd_gotoxy(14,0);
		   itoa(t, buf, 10);
		   lcd_puts(buf);
		   }
		   // Anzeige Humidity
		   lcd_gotoxy(0,1);
		   humid_val.i = abs(humid_val.i);
		   lcd_puts("H:");
		   itoa(humid_val.i/100, buf, 10);
		   lcd_gotoxy(4,1);
		   lcd_puts(buf);
		   lcd_gotoxy(6,1);
		   lcd_puts(buf);
		   lcd_gotoxy(6,1);
		   lcd_puts(".");
		   		if((humid_val.i%100) < 10)
				{
				lcd_puts("0");
		        itoa(humid_val.i%100, buf, 10);
		        lcd_puts(buf);
		        }
				else
				{
		        itoa(humid_val.i%100, buf, 10);
		        lcd_puts(buf);
				}
		   lcd_gotoxy(10,1);
		   lcd_puts("%");
		   lcd_gotoxy(12,1);
		   lcd_puts("*<");
		   lcd_gotoxy(14,1);
		   itoa(h, buf, 10);
		   lcd_puts(buf);
    	   }

		if (( s == (-1)) && ( o == (1)) && ( u == (1)))
		   {
		   // Anzeige Temperatur
		  if(temp_val.i/100 >= 10)
           {
		   lcd_gotoxy(4,0);
		   itoa(temp_val.i/100, buf, 10);
		   lcd_puts(buf);
		   lcd_gotoxy(6,0);
		   lcd_puts(".");
		   		if((temp_val.i%100) < 10)
				{
				lcd_puts("0");
		        itoa(temp_val.i%100, buf, 10);
		        lcd_puts(buf);
		        }
				else
				{
		        itoa(temp_val.i%100, buf, 10);
		        lcd_puts(buf);
				}
		   lcd_gotoxy(10,0);
		   lcd_puts("C");
		   lcd_gotoxy(13,0);
		   lcd_puts(">");
		   lcd_gotoxy(14,0);
		   itoa(t, buf, 10);
		   lcd_puts(buf);
		   }
		  if(temp_val.i/100 <= 9)
           {
		   lcd_gotoxy(5,0);
		   itoa(temp_val.i/100, buf, 10);
		   lcd_puts(buf);
		   lcd_gotoxy(6,0);
		   lcd_puts(".");
		   		if((temp_val.i%100) < 10)
				{
				lcd_puts("0");
		        itoa(temp_val.i%100, buf, 10);
		        lcd_puts(buf);
		        }
				else
				{
		        itoa(temp_val.i%100, buf, 10);
		        lcd_puts(buf);
				}
		   lcd_gotoxy(10,0);
		   lcd_puts("C");
		   lcd_gotoxy(13,0);
		   lcd_puts(">");
		   lcd_gotoxy(14,0);
		   itoa(t, buf, 10);
		   lcd_puts(buf);
		   }
		   // Anzeige Humidity
		   lcd_gotoxy(0,1);
		   humid_val.i = abs(humid_val.i);
		   lcd_puts("H:");
		   itoa(humid_val.i/100, buf, 10);
		   lcd_gotoxy(4,1);
		   lcd_puts(buf);
		   lcd_gotoxy(6,1);
		   lcd_puts(buf);
		   lcd_gotoxy(6,1);
		   lcd_puts(".");
		   		if((humid_val.i%100) < 10)
				{
				lcd_puts("0");
		        itoa(humid_val.i%100, buf, 10);
		        lcd_puts(buf);
		        }
				else
				{
		        itoa(humid_val.i%100, buf, 10);
		        lcd_puts(buf);
				}
		   lcd_gotoxy(10,1);
		   lcd_puts("%");
		   lcd_gotoxy(12,1);
		   lcd_puts("*>");
		   lcd_gotoxy(14,1);
		   itoa(h, buf, 10);
		   lcd_puts(buf);
    	   }
		
		if (( s == (1)) && ( o == (-1)) && ( u == (1)))
		   {
		   // Anzeige Temperatur
		  if(temp_val.i/100 >= 10)
           {
		   lcd_gotoxy(4,0);
		   itoa(temp_val.i/100, buf, 10);
		   lcd_puts(buf);
		   lcd_gotoxy(6,0);
		   lcd_puts(".");
		   		if((temp_val.i%100) < 10)
				{
				lcd_puts("0");
		        itoa(temp_val.i%100, buf, 10);
		        lcd_puts(buf);
		        }
				else
				{
		        itoa(temp_val.i%100, buf, 10);
		        lcd_puts(buf);
				}
		   lcd_gotoxy(10,0);
		   lcd_puts("C");
		   lcd_gotoxy(12,0);
		   lcd_puts("*<");
		   lcd_gotoxy(14,0);
		   itoa(t, buf, 10);
		   lcd_puts(buf);
		   }
		  if(temp_val.i/100 <= 9)
           {
		   lcd_gotoxy(5,0);
		   itoa(temp_val.i/100, buf, 10);
		   lcd_puts(buf);
		   lcd_gotoxy(6,0);
		   lcd_puts(".");
		   		if((temp_val.i%100) < 10)
				{
				lcd_puts("0");
		        itoa(temp_val.i%100, buf, 10);
		        lcd_puts(buf);
		        }
				else
				{
		        itoa(temp_val.i%100, buf, 10);
		        lcd_puts(buf);
				}
		   lcd_gotoxy(10,0);
		   lcd_puts("C");
		   lcd_gotoxy(12,0);
		   lcd_puts("*<");
		   lcd_gotoxy(14,0);
		   itoa(t, buf, 10);
		   lcd_puts(buf);
		   }
		   // Anzeige Humidity
		   lcd_gotoxy(0,1);
		   humid_val.i = abs(humid_val.i);
		   lcd_puts("H:");
		   itoa(humid_val.i/100, buf, 10);
		   lcd_gotoxy(4,1);
		   lcd_puts(buf);
		   lcd_gotoxy(6,1);
		   lcd_puts(buf);
		   lcd_gotoxy(6,1);
		   lcd_puts(".");
		   		if((humid_val.i%100) < 10)
				{
				lcd_puts("0");
		        itoa(humid_val.i%100, buf, 10);
		        lcd_puts(buf);
		        }
				else
				{
		        itoa(humid_val.i%100, buf, 10);
		        lcd_puts(buf);
				}
		   lcd_gotoxy(10,1);
		   lcd_puts("%");
		   lcd_gotoxy(13,1);
		   lcd_puts(">");
		   lcd_gotoxy(14,1);
		   itoa(h, buf, 10);
		   lcd_puts(buf);
    	   }
		   
		if (( s == (1)) && ( o == (1)) && ( u == (-1)))
		   {
		   // Anzeige Temperatur
		  if(temp_val.i/100 >= 10)
           {
		   lcd_gotoxy(4,0);
		   itoa(temp_val.i/100, buf, 10);
		   lcd_puts(buf);
		   lcd_gotoxy(6,0);
		   lcd_puts(".");
		   		if((temp_val.i%100) < 10)
				{
				lcd_puts("0");
		        itoa(temp_val.i%100, buf, 10);
		        lcd_puts(buf);
		        }
				else
				{
		        itoa(temp_val.i%100, buf, 10);
		        lcd_puts(buf);
				}
		   lcd_gotoxy(10,0);
		   lcd_puts("C");
		   lcd_gotoxy(12,0);
		   lcd_puts("*>");
		   lcd_gotoxy(14,0);
		   itoa(t, buf, 10);
		   lcd_puts(buf);
		   }
		  if(temp_val.i/100 <= 9)
           {
		   lcd_gotoxy(5,0);
		   itoa(temp_val.i/100, buf, 10);
		   lcd_puts(buf);
		   lcd_gotoxy(6,0);
		   lcd_puts(".");
		   		if((temp_val.i%100) < 10)
				{
				lcd_puts("0");
		        itoa(temp_val.i%100, buf, 10);
		        lcd_puts(buf);
		        }
				else
				{
		        itoa(temp_val.i%100, buf, 10);
		        lcd_puts(buf);
				}
		   lcd_gotoxy(10,0);
		   lcd_puts("C");
		   lcd_gotoxy(12,0);
		   lcd_puts("*>");
		   lcd_gotoxy(14,0);
		   itoa(t, buf, 10);
		   lcd_puts(buf);
		   }
		   // Anzeige Humidity
		   lcd_gotoxy(0,1);
		   humid_val.i = abs(humid_val.i);
		   lcd_puts("H:");
		   itoa(humid_val.i/100, buf, 10);
		   lcd_gotoxy(4,1);
		   lcd_puts(buf);
		   lcd_gotoxy(6,1);
		   lcd_puts(buf);
		   lcd_gotoxy(6,1);
		   lcd_puts(".");
		   		if((humid_val.i%100) < 10)
				{
				lcd_puts("0");
		        itoa(humid_val.i%100, buf, 10);
		        lcd_puts(buf);
		        }
				else
				{
		        itoa(humid_val.i%100, buf, 10);
		        lcd_puts(buf);
				}
		   lcd_gotoxy(10,1);
		   lcd_puts("%");
		   lcd_gotoxy(13,1);
		   lcd_puts("<");
		   lcd_gotoxy(14,1);
		   itoa(h, buf, 10);
		   lcd_puts(buf);
    	   }
		
		if (( s == (-1)) && ( o == (1)) && ( u == (-1)))
		   {
		   // Anzeige Temperatur
		  if(temp_val.i/100 >= 10)
           {
		   lcd_gotoxy(4,0);
		   itoa(temp_val.i/100, buf, 10);
		   lcd_puts(buf);
		   lcd_gotoxy(6,0);
		   lcd_puts(".");
		   		if((temp_val.i%100) < 10)
				{
				lcd_puts("0");
		        itoa(temp_val.i%100, buf, 10);
		        lcd_puts(buf);
		        }
				else
				{
		        itoa(temp_val.i%100, buf, 10);
		        lcd_puts(buf);
				}
		   lcd_gotoxy(10,0);
		   lcd_puts("C");
		   lcd_gotoxy(13,0);
		   lcd_puts(">");
		   lcd_gotoxy(14,0);
		   itoa(t, buf, 10);
		   lcd_puts(buf);
		   }
		  if(temp_val.i/100 <= 9)
           {
		   lcd_gotoxy(5,0);
		   itoa(temp_val.i/100, buf, 10);
		   lcd_puts(buf);
		   lcd_gotoxy(6,0);
		   lcd_puts(".");
		   		if((temp_val.i%100) < 10)
				{
				lcd_puts("0");
		        itoa(temp_val.i%100, buf, 10);
		        lcd_puts(buf);
		        }
				else
				{
		        itoa(temp_val.i%100, buf, 10);
		        lcd_puts(buf);
				}
		   lcd_gotoxy(10,0);
		   lcd_puts("C");
		   lcd_gotoxy(13,0);
		   lcd_puts(">");
		   lcd_gotoxy(14,0);
		   itoa(t, buf, 10);
		   lcd_puts(buf);
		   }
		   // Anzeige Humidity
		   lcd_gotoxy(0,1);
		   humid_val.i = abs(humid_val.i);
		   lcd_puts("H:");
		   itoa(humid_val.i/100, buf, 10);
		   lcd_gotoxy(4,1);
		   lcd_puts(buf);
		   lcd_gotoxy(6,1);
		   lcd_puts(buf);
		   lcd_gotoxy(6,1);
		   lcd_puts(".");
		   		if((humid_val.i%100) < 10)
				{
				lcd_puts("0");
		        itoa(humid_val.i%100, buf, 10);
		        lcd_puts(buf);
		        }
				else
				{
		        itoa(humid_val.i%100, buf, 10);
		        lcd_puts(buf);
				}
		   lcd_gotoxy(10,1);
		   lcd_puts("%");
		   lcd_gotoxy(12,1);
		   lcd_puts("*<");
		   lcd_gotoxy(14,1);
		   itoa(h, buf, 10);
		   lcd_puts(buf);
    	   }
		
		if (( s == (-1)) && ( o == (-1)) && ( u == (1)))
		   {
		   // Anzeige Temperatur
		  if(temp_val.i/100 >= 10)
           {
		   lcd_gotoxy(4,0);
		   itoa(temp_val.i/100, buf, 10);
		   lcd_puts(buf);
		   lcd_gotoxy(6,0);
		   lcd_puts(".");
		   		if((temp_val.i%100) < 10)
				{
				lcd_puts("0");
		        itoa(temp_val.i%100, buf, 10);
		        lcd_puts(buf);
		        }
				else
				{
		        itoa(temp_val.i%100, buf, 10);
		        lcd_puts(buf);
				}
		   lcd_gotoxy(10,0);
		   lcd_puts("C");
		   lcd_gotoxy(13,0);
		   lcd_puts("<");
		   lcd_gotoxy(14,0);
		   itoa(t, buf, 10);
		   lcd_puts(buf);
		   }
		  if(temp_val.i/100 <= 9)
           {
		   lcd_gotoxy(5,0);
		   itoa(temp_val.i/100, buf, 10);
		   lcd_puts(buf);
		   lcd_gotoxy(6,0);
		   lcd_puts(".");
		   		if((temp_val.i%100) < 10)
				{
				lcd_puts("0");
		        itoa(temp_val.i%100, buf, 10);
		        lcd_puts(buf);
		        }
				else
				{
		        itoa(temp_val.i%100, buf, 10);
		        lcd_puts(buf);
				}
		   lcd_gotoxy(10,0);
		   lcd_puts("C");
		   lcd_gotoxy(13,0);
		   lcd_puts("<");
		   lcd_gotoxy(14,0);
		   itoa(t, buf, 10);
		   lcd_puts(buf);
		   }
		   // Anzeige Humidity
		   lcd_gotoxy(0,1);
		   humid_val.i = abs(humid_val.i);
		   lcd_puts("H:");
		   itoa(humid_val.i/100, buf, 10);
		   lcd_gotoxy(4,1);
		   lcd_puts(buf);
		   lcd_gotoxy(6,1);
		   lcd_puts(buf);
		   lcd_gotoxy(6,1);
		   lcd_puts(".");
		   		if((humid_val.i%100) < 10)
				{
				lcd_puts("0");
		        itoa(humid_val.i%100, buf, 10);
		        lcd_puts(buf);
		        }
				else
				{
		        itoa(humid_val.i%100, buf, 10);
		        lcd_puts(buf);
				}
		   lcd_gotoxy(10,1);
		   lcd_puts("%");
		   lcd_gotoxy(12,1);
		   lcd_puts("*>");
		   lcd_gotoxy(14,1);
		   itoa(h, buf, 10);
		   lcd_puts(buf);
    	   }
		   
		}
			
	}		
/*	 //     b Wechsel zwischen Humidity und Temperatur bei 3 Taster	!!!
		if( get_key_long( 1<<KEY0 ) && get_key_long( 1<<KEY1 ))
	{
        s = s * (- 1);
		s = EEWriteByte(2, s);
    }
*/	
	
	if( get_key_short( 1<<KEY3 ))  //  b Wechsel zwischen Humidity und Temperatur bei 4 Taster	!!!
	{
		s = s * (- 1);
		s = EEWriteByte(2, s);
    }

	if( get_key_press( 1<<KEY3 ))  //  Umschalten von "normaler Anzeige" auf "Error Anzeige" bei 4 Taster	!!!
	{
        au = au * (- 1);
    }
	
	 //     o  <  > oben		
	if( (s == 1) && get_key_press( 1<<KEY2 ))
	{
		o = o * (- 1);
		o = EEWriteByte(3, o);
    }
	if( (s == (-1)) && get_key_press( 1<<KEY2 ) )
	{
		u = u * (- 1);
		u = EEWriteByte(4, u);

    }
		
		//   Temperatur einstellen:
 	 if ( s == (1))
	    if( get_key_press( 1<<KEY0 ) || get_key_rpt( 1<<KEY0 )) 
     {
        t = (t + 1);
        if(t > TMaximum)
        t = TMaximum;
		t = EEWriteByte(0, t);
     }
	 
      if ( s == (1))
        if( get_key_press( 1<<KEY1 ) || get_key_rpt( 1<<KEY1 )) 
     {
        t = (t - 1);
        if(t < TMinimum)
        t = TMinimum;
		t = EEWriteByte(0, t);
     }
	 
	 
	 	//     Humidity einstellen:
      if ( s == (-1))
        if( get_key_press( 1<<KEY0 ) || get_key_rpt( 1<<KEY0 )) 
     {
        h = (h + 1);
        if(h > HMaximum)
        h = HMaximum;
		h = EEWriteByte(1, h);
     }
      if ( s == (-1)) 
        if( get_key_press( 1<<KEY1 ) || get_key_rpt( 1<<KEY1 )) 
     {
        h = (h - 1);
        if(h < HMinimum)
        h = HMinimum;
		h = EEWriteByte(1, h);
     }
	 
	 
	  // Grer als.....
	if ((o == (1) && u == (1)))
	{
	    if (temp_val.i/100 > t || humid_val.i/100 > h)
		{
		 DDRB |= (1 << DDB1); // Setzen
		}
		
		else
		{
		DDRB &= ~(1 << DDB1); // Rcksetzen
		}
	}
	 
	  // Kleiner als.....
	if ((o == (-1) && u == (-1)))
	{
	    if (temp_val.i/100 < t || humid_val.i/100 < h)
		{
		 DDRB |= (1 << DDB1); // Setzen
		}
		
		else
		{
		DDRB &= ~(1 << DDB1); // Rcksetzen
		}
	}
	 
	  // Grer/Kleiner als.....
	if ((o == (1) && u == (-1)))
	{
	    if (temp_val.i/100 > t || humid_val.i/100 < h)
		{
		 DDRB |= (1 << DDB1); // Setzen
		}
		
		else
		{
		DDRB &= ~(1 << DDB1); // Rcksetzen
		}
	}
     
	  // Kleiner/Grer als.....
	if ((o == (-1) && u == (1)))
	{
	    if (temp_val.i/100 < t || humid_val.i/100 > h)
		{
		 DDRB |= (1 << DDB1); // Setzen
		}
		
		else
		{
		DDRB &= ~(1 << DDB1); // Rcksetzen
		}
		
	}
		wait();		   
	}
}
